package com.findwise.hydra.stage.tika;

import java.io.IOException;
import java.util.List;

import org.apache.tika.exception.TikaException;
import org.apache.tika.parser.AutoDetectParser;
import org.apache.tika.parser.Parser;
import org.xml.sax.SAXException;

import com.findwise.hydra.common.DocumentFile;
import com.findwise.hydra.local.LocalDocument;
import com.findwise.hydra.stage.AbstractProcessStage;
import com.findwise.hydra.stage.ProcessException;
import com.findwise.hydra.stage.RequiredArgumentMissingException;
import com.findwise.hydra.stage.Stage;
import com.findwise.hydra.stage.tika.utils.TikaUtils;

/**
 * @author jwestberg
 */
@Stage(description="Stage that fetches any files attached to the document being processed and parses them with Tika. Any fields found by Tika will be stored in <filename>_*")
public class TikaStage extends AbstractProcessStage {
	
	static private Parser parser = new AutoDetectParser();

	@Override
	public void process(LocalDocument doc) throws ProcessException { 
		try {
			List<String> files = getRemotePipeline().getFileNames(doc.getID());
			for(String fileName : files) {
				DocumentFile df = getRemotePipeline().getFile(fileName, doc.getID());
				TikaUtils.enrichDocumentWithFileContents(doc, fileName.replace('.', '_')+"_", df.getStream(), parser);
			}
		} catch (IOException e) {
			throw new ProcessException("Failed opening or reading from stream", e);
		} catch (SAXException e) {
			throw new ProcessException("Failed parsing document", e);
		} catch (TikaException e) {
			throw new ProcessException("Got exception from Tika", e);
		}
	}

	@Override
	public void init() throws RequiredArgumentMissingException { }

}
